﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Security.Cryptography;

namespace Buckets
{
    public enum BucketOperations
    {
        [Description ]
        Empty=0,
        Fill=1,
        Pour=2
    } ;

    public class Configuration
    {
        public Configuration Parent { get; set; }
        public Instance Instance { get; private set; }
        public int Count { get { return Volumes.Length; } }
        public byte[] Volumes { get; set; }
        public byte[] Capacities { get { return Instance.Capacities; } }
        public BucketOperations CreateOperation { get; set; }
        public int Step {get { return Parent==null?0:Parent.Step + 1;}}

        public Configuration(Instance instance)
        {
            Parent = null;
            Instance = instance;
            Volumes = new byte[instance.Count];

            for (var i = 0; i < Count; i++)
            {
                Volumes[i] = instance.Volumes[i];
            }
        }

        public Configuration(Configuration config)
        {
            Parent = config;
            Instance = config.Instance;
            Volumes = new byte[config.Count];
            for (var i = 0; i < Count; i++)
            {
                Volumes[i] = config.Volumes[i];
            }

        }

        /// <summary>
        /// Creates deep copy of Configuration
        /// </summary>
        /// <returns></returns>
        public Configuration Clone()
        {
            var newConfig = new Configuration(this);
            return newConfig;
        }

        public override string ToString()
        {
            string returnValue = String.Format("{0}: {1} ",Instance.ID, Step);
            returnValue = Volumes.Aggregate(returnValue, (current, item) => current + (item + " "));

            return returnValue;
        }

        /// <summary>
        /// Creates MD5 hash as a string.
        /// </summary>
        /// <returns>Returns MD5 hash string</returns>
        public string GetMD5Hash()
        {
            using (MD5 md5 = MD5.Create())
            {
                byte[] byteHashed = md5.ComputeHash(Volumes);
                return  Convert.ToBase64String(byteHashed);
            }

        }

        public string SolutionToString()
        {
            string returnValue = String.Format("{0}: ", Step);
            returnValue = Volumes.Aggregate(returnValue, (current, item) => current + (item + " "));

            if (Parent == null) return returnValue;

            return String.Format("{0}\n{1}\n{2}", Parent.SolutionToString(), CreateOperation, returnValue);

        }

    }
}
