﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Buckets
{
    public interface IConfigCollection
    {
        bool Add(Configuration config);
        void Add(IEnumerable<Configuration> configurations);
        Configuration Get();
        int Count { get; }
        int NumOfStates { get; }

    }

    public class ConfigQueue : IConfigCollection
    {
        private HashSet<string> _configHashs;
        private Queue<Configuration> _configQueue;
        
        public ConfigQueue()
        {
            _configHashs = new HashSet<string>();
            _configQueue = new Queue<Configuration>();
        }
        
        public bool Add(Configuration config)
        {
            if (_configHashs.Add(config.GetMD5Hash()))
            {
                _configQueue.Enqueue(config);
                return true;
            }
            return false;
        }

        public void Add(IEnumerable<Configuration> configurations)
        {
            foreach (var config in configurations)
            {
                Add(config);
            }
        }

        public Configuration Get()
        {
            return _configQueue.Dequeue();
        }

        public int Count
        {
            get { return _configQueue.Count; }
        }

        public int NumOfStates
        {
            get { return _configHashs.Count; }
        }

    }

    public class ConfigPriorityQueue : IConfigCollection
    {
        private HashSet<string> _configHashs;
        private LinkedList<Configuration> _linkedList;
        private IComparer<Configuration> _comparer;

        public ConfigPriorityQueue(IComparer<Configuration> comparer)
        {
            _configHashs = new HashSet<string>();
            _linkedList = new LinkedList<Configuration>();
            _comparer = comparer;
        }

        public bool Add(Configuration config)
        {
            if (_configHashs.Add(config.GetMD5Hash()))
            {
                if (_linkedList.First == null)
                {
                    _linkedList.AddFirst(config);
                    return true;
                }
                
                var current = _linkedList.First;
                while (current.Next != null)
                {
                    if (_comparer.Compare(current.Value, current.Next.Value) <= 0)
                    {
                        current = current.Next;
                        continue;
                    }
                    break;
                }

                _linkedList.AddAfter(current, config);

                return true;
            }
            return false;
        }

        public void Add(IEnumerable<Configuration> configurations)
        {
            foreach (var config in configurations)
            {
                Add(config);
            }
        }

        public Configuration Get()
        {
            var returnValue = _linkedList.First.Value;
            _linkedList.RemoveFirst();
            return returnValue;

        }

        public int Count
        {
            get { return _linkedList.Count; }
        }

        public int NumOfStates
        {
            get { return _configHashs.Count; }
        }
    }

}
