﻿using System.Collections.Generic;
using System.Linq;

namespace Buckets
{
    /// <summary>
    /// Buckets problem instance
    /// </summary>
    public class Instance
    {
        public int ID { get; private set; }
        public int Count { get; private set; }

        public byte[] Capacities { get; set; }
        public byte[] Volumes { get; set; }
        public byte[] Finals { get; set; }

        private List<Bucket> _items;
        public IList<Bucket> Items
        {
            get { return _items.AsReadOnly(); }
        }

        public Instance(int id, int count, List<Bucket> items)
        {
            ID = id;
            Count = count;
            _items = items;

            Capacities = new byte[Count];
            Volumes = new byte[Count];
            Finals = new byte[Count];

            for (int i = 0; i < items.Count; i++)
            {
                Capacities[i] = items[i].Capacity;
                Volumes[i] = items[i].Volume;
                Finals[i] = items[i].Final;
            }
        }

        public void SortItems(IComparer<Bucket> comparer)
        {
            _items = _items.OrderBy(item => item, comparer).ToList();
        }

        public bool IsSolution(byte[] volumes)
        {
            return !Volumes.Where((t, i) => volumes[i] != Finals[i]).Any();
        }

        public override string ToString()
        {
            string returnValue = string.Format("{0} {1} ", ID, Count);
            returnValue = Items.Aggregate(returnValue, (current, item) => current + (item.Capacity + " "));
            returnValue = Items.Aggregate(returnValue, (current, item) => current + (item.Volume + " "));
            returnValue = Items.Aggregate(returnValue, (current, item) => current + (item.Final + " "));

            return returnValue;
        }

    }
}