﻿using System;
using System.Collections.Generic;
using System.IO;

namespace Buckets.Console
{
    class Program
    {
        public static Dictionary<int, Instance> ReadInstances(string filePath)
        {
            var instances = new Dictionary<int, Instance>();

            string line;
            StreamReader sr = null;
            try
            {
                sr = new StreamReader(filePath);

                while (!String.IsNullOrEmpty(line = sr.ReadLine()))
                {
                    var values = line.Split(new[]{' '}, StringSplitOptions.RemoveEmptyEntries);
                    var id = byte.Parse(values[0]);
                    var count = byte.Parse(values[1]);
                    var items = new List<Bucket>(count);

                    for (int i = 2; i < count + 2; i++)
                    {
                        var c = byte.Parse(values[i]);
                        var aV = byte.Parse(values[i + count]);
                        var fV = byte.Parse(values[i + 2 * count]);
                        items.Add(new Bucket() { Capacity = c, Volume = aV, Final = fV});
                    }

                    var instance = new Instance(id, count, items);
                    instances.Add(instance.ID, instance);
                }
            }
            finally
            {
                if (sr != null) sr.Dispose();
            }

            return instances;
        }

        static void Main(string[] args)
        {
            System.Console.WriteLine("====== The water jug problem ======\n");

            #region file paths define
            const string instancesExtension = @".inst.dat";
            const string solutionsExtension = @".sol.dat";
            const string resultsExtension = @".res.dat";
            const string instancesDirectoryName = @"instances\";
            const string solutionsDirectoryName = @"solutions\";
            const string resultsDirectoryName = @"results\";
            #endregion

            var instancesfilePaths = Directory.GetFiles(instancesDirectoryName, "*" + instancesExtension);

            foreach (var insFilePath in instancesfilePaths)
            {
                #region read instances and solutions

                var insName =
                    insFilePath.Remove(insFilePath.Length - instancesExtension.Length,
                                        instancesExtension.Length).Remove(0, instancesDirectoryName.Length);
                Dictionary<int, Instance> instances = ReadInstances(insFilePath);

                #endregion

                Solver solver = new Solver();
                StreamWriter sw = new StreamWriter(resultsDirectoryName + insName + resultsExtension);

                foreach (var instance in instances)
                {
                    //System.Console.WriteLine(instance.Value);

                    #region Solve BFS
                    //var solutionBFS = solver.Solve(instance.Value, Methods.BFS);
                    //System.Console.WriteLine(solutionBFS);

                    #endregion

                    
                    #region Solve Heuristic
                    //var solutionH1 = solver.Solve(instance.Value, Methods.Volume);
                    //System.Console.WriteLine(solutionH1);

                    //var solutionH2 = solver.Solve(instance.Value, Methods.PathLength);
                    //System.Console.WriteLine(solutionH2);

                    //var solutionH3 = solver.Solve(instance.Value, Methods.SumVolume);
                    //System.Console.WriteLine(solutionH3);

                    var solutionH4 = solver.Solve(instance.Value, Methods.Combine1);
                    System.Console.WriteLine(solutionH4);

                    //var solutionH5 = solver.Solve(instance.Value, Methods.Combine2);
                    //System.Console.WriteLine(solutionH5);

                    #endregion

                    #region Write results

                    //sw.WriteLine("{0}, {1}, {2}, {3}, {4}, {5}, {6}, {7},{8},{9},{10},{11},{12}", solutionBFS.ID,
                    //             solutionBFS.NumOfNodes,
                    //             solutionBFS.Steps, solutionH1.NumOfNodes, solutionH1.Steps, solutionH2.NumOfNodes,
                    //             solutionH2.Steps, solutionH3.NumOfNodes, solutionH3.Steps, solutionH4.NumOfNodes,
                    //             solutionH4.Steps, solutionH5.NumOfNodes, solutionH5.Steps);
                    #endregion

                    System.Console.WriteLine();
                }

                sw.Dispose();
            }

            System.Console.ReadLine();
        }
    }

}
